from __future__ import annotations

from functools import partial
from typing import TYPE_CHECKING, Iterator, List, Optional, cast

from root.generated.openapi_client.api.objectives_api import ObjectivesApi
from root.generated.openapi_client.models.evaluator_execution_functions_request import (
    EvaluatorExecutionFunctionsRequest,
)
from root.generated.openapi_client.models.objective import Objective as OpenApiObjective
from root.generated.openapi_client.models.objective_execution_request import ObjectiveExecutionRequest
from root.generated.openapi_client.models.objective_list import ObjectiveList
from root.generated.openapi_client.models.objective_request import ObjectiveRequest
from root.generated.openapi_client.models.validator_execution_result import ValidatorExecutionResult

from .generated.openapi_client import ApiClient
from .skills import Skills
from .utils import iterate_cursor_list

if TYPE_CHECKING:
    from .validators import Validator


class Objective(OpenApiObjective):
    """Wrapper for a single Objective.

    For available attributes, please check the (automatically
    generated) superclass documentation.
    """

    _client: ApiClient

    @classmethod
    def _wrap(cls, apiobj: OpenApiObjective, client: ApiClient) -> "Objective":
        if not isinstance(apiobj, OpenApiObjective):
            raise ValueError(f"Wrong instance in _wrap: {apiobj!r}")
        obj = cast(Objective, apiobj)
        obj.__class__ = cls
        obj._client = client
        return obj

    def run(
        self,
        *,
        response: str,
        request: Optional[str] = None,
        contexts: Optional[List[str]] = None,
        functions: Optional[List[EvaluatorExecutionFunctionsRequest]] = None,
    ) -> ValidatorExecutionResult:
        """
        Run all validators associated with the objective.

        Args:

          response: LLM output.

          request: The prompt sent to the LLM. Optional.

          contexts: Optional documents passed to RAG evaluators

        """
        api_instance = ObjectivesApi(self._client)
        skill_execution_request = ObjectiveExecutionRequest(
            request=request,
            response=response,
            contexts=contexts,
            functions=functions,
        )
        return api_instance.objectives_objectives_execute_create(
            objective_id=self.id, objective_execution_request=skill_execution_request
        )


class Objectives:
    """Objectives API

    Note:

      The construction of the API instance should be handled by
      accesing an attribute of a :class:`root.client.RootSignals` instance.
    """

    def __init__(self, client: ApiClient):
        self.client = client

    def create(self, *, intent: Optional[str] = None, validators: Optional[List[Validator]] = None) -> Objective:
        """Create a new objective and return its ID.

        Args:

          intent: The intent of the objective.

          validators: An optional list of validators.

        """
        skills = Skills(self.client)
        request = ObjectiveRequest(
            intent=intent,
            validators=[validator._to_request(skills) for validator in validators or []],
        )
        api_instance = ObjectivesApi(self.client)
        objective = api_instance.objectives_create(objective_request=request)
        return self.get(objective.id)

    def get(self, objective_id: str) -> Objective:
        """
        Get an objective by ID.

        Args:

          objective_id: The objective to be fetched.

        """
        api_instance = ObjectivesApi(self.client)
        return Objective._wrap(api_instance.objectives_retrieve(id=objective_id), self.client)

    def delete(self, objective_id: str) -> None:
        """
        Delete the objective from the registry.

        Args:

          objective_id: The objective to be deleted.

        """
        api_instance = ObjectivesApi(self.client)
        return api_instance.objectives_destroy(id=objective_id)

    def list(self, *, intent: Optional[str] = None, limit: int = 100) -> Iterator[ObjectiveList]:
        """Iterate through the objectives.

        Note:

          The call will list only publicly available global objectives and
          those objectives available to the organzation(s) of the user.

        Args:
          intent: Specific intent the returned objectives must match.
          limit: Number of entries to iterate through at most.

        """
        api_instance = ObjectivesApi(self.client)
        yield from iterate_cursor_list(partial(api_instance.objectives_list, intent=intent), limit=limit)

    def run(
        self,
        objective_id: str,
        *,
        response: str,
        request: Optional[str] = None,
        contexts: Optional[List[str]] = None,
        functions: Optional[List[EvaluatorExecutionFunctionsRequest]] = None,
    ) -> ValidatorExecutionResult:
        """
        Run all validators associated with an objective.

        Args:

          response: LLM output.

          request: The prompt sent to the LLM. Optional.

          contexts: Optional documents passed to RAG evaluators


        """
        api_instance = ObjectivesApi(self.client)
        skill_execution_request = ObjectiveExecutionRequest(
            request=request,
            response=response,
            contexts=contexts,
            functions=functions,
        )
        return api_instance.objectives_objectives_execute_create(
            objective_id=objective_id, objective_execution_request=skill_execution_request
        )



from root import RootSignals
from root.validators import Validator

# Connect to the Root Signals API
client = RootSignals()

# Create an objective which describes what we are trying to do
objective = client.objectives.create(
    intent="Child-safe clear response",
    validators=[
        Validator(evaluator_name="Clarity", threshold=0.2),
        Validator(evaluator_name="Safety for Children", threshold=0.3),
    ],
)


llm_response = "Some LLM response I got from my custom LLM pipeline."
response = objective.run(response=llm_response)

print(response)
# validation:
#   validation_results: [
#     evaluator_name: Clarity
#     result: 0.5
#     is_valid: true
#     ...
#   ]



from root import RootSignals

# Connect to the Root Signals API
client = RootSignals()

# Add a self-hosted model using Ollama
model = client.models.create(
    name="ollama/llama3",
    # URL pointing to the model's endpoint. Replace this with your own endpoint.
    url="https://d65e-88-148-175-2.ngrok-free.app",
)

# Use the model in a skill
skill = client.skills.create(
    name="My model test", prompt="Hello, my model!", model="ollama/llama3"
)



from unittest.mock import patch

import pytest
from root.client import RootSignals
from root.generated.openapi_client.models.skill_test_output import SkillTestOutput
from root.skills import CalibrateBatchParameters


@pytest.fixture
def mock_skills_calibrate_api():
    with patch("root.generated.openapi_client.api.skills_api.SkillsApi.skills_calibrate_create") as mock:
        yield mock


def test_calibrate_evaluator_batch(mock_skills_calibrate_api):
    mock_skills_calibrate_api.return_value = [
        SkillTestOutput(result={"score": 0.8, "expected_score": 0.75}, variables={}),
        SkillTestOutput(result={"score": 0.9, "expected_score": 0.55}, variables={}),
    ]

    client = RootSignals(api_key="fake")

    params = [
        CalibrateBatchParameters(
            name="With gpt-4",
            prompt="What is the weather today?",
            model="gpt-4",
            pii_filter=False,
            reference_variables=None,
            input_variables=None,
            data_loaders=None,
        ),
        CalibrateBatchParameters(
            name="With gpt-3.5-turbo",
            prompt="What is the weather today?",
            model="gpt-3.5-turbo",
            pii_filter=False,
            reference_variables=None,
            input_variables=None,
            data_loaders=None,
        ),
    ]

    result = client.evaluators.calibrate_batch(
        evaluator_definitions=params, test_data=[["0.4", "LLM output"], ["0.6", "LLM output 2"]]
    )

    assert result.rms_errors_model == {
        "gpt-4": pytest.approx(0.2499, rel=1e-3),
        "gpt-3.5-turbo": pytest.approx(0.2499, rel=1e-3),
    }

    assert result.mae_errors_model == {
        "gpt-4": pytest.approx(0.2, rel=1e-3),
        "gpt-3.5-turbo": pytest.approx(0.2, rel=1e-3),
    }

    assert result.rms_errors_prompt == {
        "What is the weather today?": pytest.approx(0.25, rel=1e-3),
    }
    assert len(result.results) == 4



from root import RootSignals
from root.skills import EvaluatorDemonstration

client = RootSignals()

# Create an evaluator
direct_language_evaluator = client.skills.create(
    name="Direct language",
    prompt="Does the {{ output }} contain weasel words?",
    intent="Is the language direct and unambiguous",
    model="gpt-3.5-turbo",
    is_evaluator=True,
)

# Run first calibration
test_result = client.evaluators.calibrate_existing(
    evaluator_id=direct_language_evaluator.id,
    test_data=[
        ["0.1", "There will probably be a meeting tomorrow"],
        ["0.1", "We probably won't need to make any major changes."],
    ],
)
print(test_result[0].result)
#  "score": 0.5,
#  "expected_score": 0.1,

# Improve the evaluator with demonstrations, penalize the "probably" weasel word
client.skills.update(
    skill_id=direct_language_evaluator.id,
    evaluator_demonstrations=[
        EvaluatorDemonstration(
            output="The project will probably be completed on time.",
            score=0.1,
        ),
        EvaluatorDemonstration(
            output="He probably knows the answer to your question.",
            score=0.1,
        ),
        EvaluatorDemonstration(
            output="It will probably rain later today.", score=0.1
        ),
    ],
)
# Run second calibration
test_result = client.evaluators.calibrate_existing(
    evaluator_id=direct_language_evaluator.id,
    test_data=[
        ["0.1", "There will probably be a meeting tomorrow"],
        ["0.1", "We probably won't need to make any major changes."],
    ],
)

# Check the results
print(test_result[0].result)
#  "score": 0.1,
#  "expected_score": 0.1,



import tempfile

from root import RootSignals
from root.skills import ReferenceVariable

client = RootSignals()

with tempfile.NamedTemporaryFile(suffix=".csv", mode="w") as fp:
    fp.write("short@example.com\nlonger-email@example.com\n")
    fp.flush()
    dataset = client.datasets.create(
        name="List of email addresses", path=fp.name, type="reference"
    )

skill = client.skills.create(
    reference_variables=[
        ReferenceVariable(dataset_id=dataset.id, name="email_dataset")
    ],
    intent="Email address list assistant.",
    name="Email address dataset chatbot",
    prompt="{{email_dataset}}",
    model="gpt-4",
)

response = skill.run(
    {
        "email_dataset": (
            "Which email has the longest domain name?"
            "Respond with just the email address."
        )
    },
)
print(response.llm_output)
# longer-email@example.com



from typing import Dict, Optional, cast

from root.generated.openapi_client.api.chats_api import ChatsApi
from root.generated.openapi_client.models.chat_detail import ChatDetail
from root.generated.openapi_client.models.chat_execution_request_request import ChatExecutionRequestRequest
from root.generated.openapi_client.models.chat_execution_result import ChatExecutionResult

from .generated.openapi_client import ApiClient


class SkillChat(ChatDetail):
    """Wrapper for a single Chat with a Skill.

    For available attributes, please check the (automatically
    generated) superclass documentation.
    """

    _client: ApiClient

    @classmethod
    def _wrap(cls, chat: ChatDetail, client: ApiClient) -> "SkillChat":
        if not isinstance(chat, ChatDetail):
            raise ValueError(f"Wrong instance in _wrap: {chat!r}")
        schat = cast(SkillChat, chat)
        schat.__class__ = cls
        schat._client = client
        return schat

    def run(
        self, variables: Optional[Dict[str, str]] = None, *, skill_version_id: Optional[str] = None
    ) -> ChatExecutionResult:
        """
        Run a skill with optional variables as an interactive chat. The chat message history is stored and can
        be referred to in following calls.

        The response is equal to the skill run response, with the addition of a chat_id key.
        """
        api_instance = ChatsApi(self._client)
        chat_execution_request = ChatExecutionRequestRequest(
            variables=variables,
            skill_version_id=skill_version_id,
        )
        return api_instance.chats_execute_create(
            chat_id=self.chat_id, chat_execution_request_request=chat_execution_request
        )



from __future__ import annotations

import os
import re
import textwrap
from functools import cached_property
from typing import TYPE_CHECKING, Optional

from .__about__ import __version__
from .generated import openapi_client
from .generated.openapi_client.configuration import Configuration as _Configuration

if TYPE_CHECKING:
    from .datasets import DataSets
    from .execution_logs import ExecutionLogs
    from .models import Models
    from .objectives import Objectives
    from .skills import Evaluators, Skills


def _get_api_key(*, dot_env: str = ".env") -> str:
    var = "ROOTSIGNALS_API_KEY"
    api_key = os.environ.get(var)
    if api_key is not None:
        return api_key
    if os.path.exists(dot_env):
        for line in open(dot_env):
            m = re.fullmatch(line, rf"^\s*{var}\s*=\s*(\S+)\s*$")
            if m is not None:
                return m.group(1)
    raise ValueError(
        textwrap.dedent("""
    Root Signals API key cannot be found.
    It can be provided in client invocation, using ROOTSIGNALS_API_KEY environment variable or .env file line
    """)
    )


class RootSignals:
    """Root Signals API Python client.

    The API key must be provided via one of the following methods - the code uses the first one that is found:

    1. as an argument to RootSignals constructor    ,
    2. environment variable `ROOTSIGNALS_API_KEY`, or
    3. .env file containing `ROOTSIGNALS_API_KEY=`

    Args:
        api_key: Root Signals API Key (if not provided from environment)
        base_url: Root Signals base URL to use (default is fine)
    """

    def __init__(
        self,
        api_key: Optional[str] = None,
        *,
        base_url: Optional[str] = None,
        _api_client: Optional[openapi_client.ApiClient] = None,
    ):
        if api_key is None:
            api_key = _get_api_key()
        if base_url is None:
            base_url = os.environ.get("ROOTSIGNALS_API_URL", "https://api.app.rootsignals.ai")
        self.base_url = base_url
        self.api_key = api_key
        self._api_client_arg = _api_client

    @cached_property
    def _api_client(self) -> openapi_client.ApiClient:
        """Get the OpenAPI client

        End users should not need to inheract with OpenAPI directly.

        Note that this call is cached for duration of the RootSignals
        instance; later calls will return the same instance.
        """
        if self._api_client_arg is not None:
            return self._api_client_arg

        api_client_configuration = _Configuration(host=self.base_url)
        api_client_configuration.api_key["publicApiKey"] = f"Api-Key {self.api_key}"
        return openapi_client.ApiClient(
            api_client_configuration, header_name="x-root-python-version", header_value=__version__
        )

    @cached_property
    def datasets(self) -> DataSets:
        """Get DataSets API"""

        from .datasets import DataSets

        return DataSets(self._api_client, self.base_url, self.api_key)

    @cached_property
    def evaluators(self) -> Evaluators:
        """Get Evaluators API"""
        from .skills import Evaluators

        return Evaluators(self._api_client)

    @cached_property
    def execution_logs(self) -> ExecutionLogs:
        """Get Execution Logs API"""
        from .execution_logs import ExecutionLogs

        return ExecutionLogs(self._api_client)

    @cached_property
    def models(self) -> Models:
        """Get Models API"""
        from .models import Models

        return Models(self._api_client)

    @cached_property
    def objectives(self) -> Objectives:
        """Get Objectives API"""
        from .objectives import Objectives

        return Objectives(self._api_client)

    @cached_property
    def skills(self) -> Skills:
        """Get Skills API"""
        from .skills import Skills

        return Skills(self._api_client)



# coding: utf-8

"""
Root Signals API

Root Signals JSON API provides a way to access Root Signals using provisioned API token

The version of the OpenAPI document: 1.0.0 (latest)
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from datetime import datetime
from typing import Any, ClassVar, Dict, List, Optional, Set, Union

from pydantic import BaseModel, ConfigDict, StrictFloat, StrictInt, StrictStr
from typing_extensions import Self

from root.generated.openapi_client.models.execution_log_details_objective import ExecutionLogDetailsObjective
from root.generated.openapi_client.models.execution_log_details_skill import ExecutionLogDetailsSkill
from root.generated.openapi_client.models.model_params import ModelParams
from root.generated.openapi_client.models.nested_user_details import NestedUserDetails
from root.generated.openapi_client.models.skill_execution_validator_result import SkillExecutionValidatorResult


class ExecutionLogDetails(BaseModel):
    """
    ExecutionLogDetails
    """  # noqa: E501

    chat_id: Optional[StrictStr]
    cost: Optional[Union[StrictFloat, StrictInt]]
    created_at: Optional[datetime]
    id: StrictStr
    justification: StrictStr
    llm_output: StrictStr
    model: StrictStr
    model_call_duration: Union[StrictFloat, StrictInt]
    model_params: Optional[ModelParams] = None
    objective: ExecutionLogDetailsObjective
    owner: NestedUserDetails
    rendered_prompt: StrictStr
    skill: ExecutionLogDetailsSkill
    validation_results: List[SkillExecutionValidatorResult]
    variables: Optional[Any]
    __properties: ClassVar[List[str]] = [
        "chat_id",
        "cost",
        "created_at",
        "id",
        "justification",
        "llm_output",
        "model",
        "model_call_duration",
        "model_params",
        "objective",
        "owner",
        "rendered_prompt",
        "skill",
        "validation_results",
        "variables",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ExecutionLogDetails from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set(
            [
                "chat_id",
                "cost",
                "created_at",
                "id",
                "justification",
                "llm_output",
                "model",
                "owner",
                "rendered_prompt",
                "validation_results",
                "variables",
            ]
        )

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of model_params
        if self.model_params:
            _dict["model_params"] = self.model_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of objective
        if self.objective:
            _dict["objective"] = self.objective.to_dict()
        # override the default output from pydantic by calling `to_dict()` of owner
        if self.owner:
            _dict["owner"] = self.owner.to_dict()
        # override the default output from pydantic by calling `to_dict()` of skill
        if self.skill:
            _dict["skill"] = self.skill.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in validation_results (list)
        _items = []
        if self.validation_results:
            for _item in self.validation_results:
                if _item:
                    _items.append(_item.to_dict())
            _dict["validation_results"] = _items
        # set to None if chat_id (nullable) is None
        # and model_fields_set contains the field
        if self.chat_id is None and "chat_id" in self.model_fields_set:
            _dict["chat_id"] = None

        # set to None if cost (nullable) is None
        # and model_fields_set contains the field
        if self.cost is None and "cost" in self.model_fields_set:
            _dict["cost"] = None

        # set to None if created_at (nullable) is None
        # and model_fields_set contains the field
        if self.created_at is None and "created_at" in self.model_fields_set:
            _dict["created_at"] = None

        # set to None if model_params (nullable) is None
        # and model_fields_set contains the field
        if self.model_params is None and "model_params" in self.model_fields_set:
            _dict["model_params"] = None

        # set to None if variables (nullable) is None
        # and model_fields_set contains the field
        if self.variables is None and "variables" in self.model_fields_set:
            _dict["variables"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ExecutionLogDetails from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "chat_id": obj.get("chat_id"),
                "cost": obj.get("cost"),
                "created_at": obj.get("created_at"),
                "id": obj.get("id"),
                "justification": obj.get("justification"),
                "llm_output": obj.get("llm_output"),
                "model": obj.get("model"),
                "model_call_duration": obj.get("model_call_duration"),
                "model_params": ModelParams.from_dict(obj["model_params"])
                if obj.get("model_params") is not None
                else None,
                "objective": ExecutionLogDetailsObjective.from_dict(obj["objective"])
                if obj.get("objective") is not None
                else None,
                "owner": NestedUserDetails.from_dict(obj["owner"]) if obj.get("owner") is not None else None,
                "rendered_prompt": obj.get("rendered_prompt"),
                "skill": ExecutionLogDetailsSkill.from_dict(obj["skill"]) if obj.get("skill") is not None else None,
                "validation_results": [
                    SkillExecutionValidatorResult.from_dict(_item) for _item in obj["validation_results"]
                ]
                if obj.get("validation_results") is not None
                else None,
                "variables": obj.get("variables"),
            }
        )
        return _obj



from functools import partial
from typing import Any, Dict, Iterator, Optional

import requests

from root.generated.openapi_client.api.datasets_api import DatasetsApi
from root.generated.openapi_client.models.data_set_create import DataSetCreate
from root.generated.openapi_client.models.data_set_list import DataSetList

from .generated.openapi_client import ApiClient
from .utils import iterate_cursor_list


class DataSets:
    """DataSets (sub) API

    Note:

      The construction of the API instance should be handled by
      accesing an attribute of a :class:`root.client.RootSignals` instance.
    """

    def __init__(self, client: ApiClient, base_url: str, api_key: str):
        self.client = client
        self.base_url = base_url
        self.api_key = api_key

    def create(
        self, *, name: Optional[str] = None, path: Optional[str] = None, type: str = "reference"
    ) -> Optional[DataSetCreate]:
        """
        Create a dataset object with the given parameters to the registry.
        If the dataset has a path, it will be uploaded to the registry.

        """

        payload: Dict[str, Any] = {"name": name, "type": type, "tags": []}
        if path:
            files = {"file": open(path, "rb")}

        # TODO Should use the generated client if the generator some day supports streaming upload
        # (Currently the underlying implementation loads file contents to memory always)
        response = requests.post(
            f"{self.base_url}/datasets/",
            headers={"Authorization": f"Api-Key {self.api_key}"},
            data=payload,
            files=files,
            timeout=120,
        )
        if not response.ok:
            raise Exception(f"create failed with status code {response.status_code} and message\n{response.text}")

        return DataSetCreate.from_dict(response.json())

    def get(self, dataset_id: str) -> DataSetList:
        """
        Get a dataset object from the registry.
        """
        api_instance = DatasetsApi(self.client)
        return api_instance.datasets_retrieve(dataset_id)

    def list(self, search_term: Optional[str] = None, *, limit: int = 100) -> Iterator[DataSetList]:
        """Iterate through the datasets.

        Args:
          limit: Number of entries to iterate through at most.

          search_term: Can be used to limit returned datasets.

        """
        api_instance = DatasetsApi(self.client)
        yield from iterate_cursor_list(partial(api_instance.datasets_list, search=search_term), limit=limit)

    def delete(self, dataset_id: str) -> None:
        """
        Delete a dataset object from the registry.
        """
        api_instance = DatasetsApi(self.client)
        return api_instance.datasets_destroy(dataset_id)



from root import RootSignals
from root.validators import Validator

# Connect to the Root Signals API
client = RootSignals()

skill = client.skills.create(
    name="My strict chatbot",
    intent="Simple Q&A chatbot",
    prompt="Provide a clear answer to the question: {{question}}",
    model="gpt-4",
    validators=[Validator(evaluator_name="Clarity", threshold=0.6)],
)

# Execute the skill
response = skill.run({"question": "What is the capital of France?"})

# Get the validation results
print(response.validation)
# {'validator_results': [{'cost': 0.000..., 'evaluator_name': 'Clarity'
# 'threshold': 0.6, is_valid': True, 'result': 0.xy}]


# Get the execution details
log = client.execution_logs.get(execution_result=response)
print(log)
# cost: 0.000...
# skill:
#   name: My Q&A chatbot
#   ...
# llm_output: "The capital of France is Paris."
# rendered_prompt: "Provide a clear answer to the question: What is ..."
# validation_results: [
#   evaluator_name: Clarity
#   result: 0.9
#   is_valid: true
#   ...
# ]
# ...

# List all execution logs
iterator = client.execution_logs.list(limit=10)
print(next(iterator))
# Note that the list result does not contain the full execution details

# cost: 0.000...
# skill:
#  name: My Q&A chatbot
#  ...
